@@ -27,6 +27,7 @@
 #include <unistd.h>
 #include <sys/types.h>
 #include <sys/wait.h>
+#include <sys/ioctl.h>
 #include <assert.h>
 #include <string.h>
 #include <errno.h>
@@ -65,18 +66,32 @@ fail:
     exit(1);
 }
 
-/* Add current argument (optarg) to given list
- * Check for its correctness
- */
-void add_arg_to_fd_list(int *list, int *list_count)
+static void add_fd_to_list(int const untrusted_cur_fd,
+                           int *const list, int *const list_count,
+                           const int *const other_list, const int *const other_list_count)
 {
-    int i, cur_fd, untrusted_cur_fd;
+    int i, cur_fd;
 
-    if (*list_count >= MAX_FDS - 1) {
-        fprintf(stderr, "Too many FDs specified\n");
-        exit(1);
+    if (*list_count >= MAX_FDS - 1)
+        errx(1, "Too many FDs specified (found %d, limit %d)",
+             *list_count, MAX_FDS - 1);
+    // check if already used for I/O in the other direction
+    for (i = 0; i < *other_list_count; i++) {
+        if (other_list[i] == untrusted_cur_fd) {
+            switch (untrusted_cur_fd) {
+            case 0:
+                errx(1, "Cannot write to stdin");
+            case 1:
+                errx(1, "Cannot read from stdout");
+            case 2:
+                errx(1, "Cannot read from stderr");
+            default:
+                errx(1,
+                     "Cannot use fd %d for both reading and writing",
+                     untrusted_cur_fd);
+            }
+        }
     }
-    untrusted_cur_fd = validate_fd_argument(optarg);
     // check if not already in list
     for (i = 0; i < *list_count; i++) {
         if (list[i] == (int)untrusted_cur_fd)
@@ -84,59 +99,46 @@ void add_arg_to_fd_list(int *list, int *list_count)
     }
     cur_fd = untrusted_cur_fd;
     /* FD sanitization end */
-    if (i == *list_count)
+    if (i == *list_count) {
+        if (is_client && cur_fd > 2 && ioctl(cur_fd, FIOCLEX))
+            err(1, "Cannot make file descriptor %d close-on-exec", cur_fd);
         list[(*list_count)++] = cur_fd;
+    }
 }
 
-void handle_opt_verify(char *untrusted_sig_path, int *list, int *list_count, int is_client)
+/* Add current argument (optarg) to given list
+ * Check for its correctness
+ */
+static void add_arg_to_fd_list(int *list, int *list_count,
+                        const int *other_list, const int *other_list_count)
+{
+    int untrusted_cur_fd = validate_fd_argument(optarg);
+    add_fd_to_list(untrusted_cur_fd, list, list_count, other_list, other_list_count);
+}
+
+static void handle_opt_verify(char **untrusted_sig_path_ptr, int *input_list, int *input_list_count,
+                              const int *output_list, const int *output_list_count)
 {
-    int i;
-    char *sig_path;
     int cur_fd;
-    int untrusted_sig_path_len;
-    int fd_path_len;
 
-    if (*list_count >= MAX_FDS - 1) {
-        fprintf(stderr, "Too many FDs used\n");
-        exit(1);
-    }
-    if (untrusted_sig_path[0] == 0) {
-        fprintf(stderr, "Invalid fd argument\n");
-        exit(1);
-    }
-    if (!strncmp(untrusted_sig_path, "/dev/fd/", 8)) {
-        cur_fd = validate_fd_argument(untrusted_sig_path + 8);
-    } else {
-        if (!is_client) {
-            fprintf(stderr, "--verify with filename allowed only on the client side\n");
-            exit(1);
-        }
+    if (is_client) {
         /* arguments on client side are trusted */
-        sig_path = untrusted_sig_path;
-        cur_fd = open(sig_path, O_RDONLY|O_CLOEXEC|O_NOCTTY);
-        if (cur_fd < 0) {
-            perror("open sig");
-            exit(1);
-        }
-        /* HACK: override original file path with FD virtual path, hope it will
-         * fit; use /dev/fd instead of /proc/self/fd because is is shorter and
-         * space is critical here (for thunderbird it must fit in place of "/tmp/data.sig") */
-        untrusted_sig_path_len = strlen(untrusted_sig_path);
-        fd_path_len = snprintf(untrusted_sig_path, untrusted_sig_path_len + 1, "/dev/fd/%d", cur_fd);
-        if (fd_path_len < 0 || fd_path_len > untrusted_sig_path_len) {
-            fprintf(stderr, "Failed to fit /dev/fd/%d in place of signature path\n", cur_fd);
-            exit(1);
-        }
-        /* leak FD intentionally - process_io will read from it */
+        char *sig_path = *untrusted_sig_path_ptr;
+        if (!strcmp(sig_path, "-"))
+            cur_fd = fcntl(0, F_DUPFD_CLOEXEC, 3);
+        else
+            cur_fd = open(sig_path, O_RDONLY|O_CLOEXEC|O_NOCTTY);
+        if (cur_fd < 0)
+            err(1, "open sig file %s", sig_path);
+        if (asprintf(untrusted_sig_path_ptr, "/dev/fd/%d", cur_fd) < 0)
+            err(1, "asprintf");
+    } else {
+        if (strncmp((*untrusted_sig_path_ptr), "/dev/fd/", 8))
+            errx(1, "--verify with filename allowed only on the client side");
+        if ((cur_fd = validate_fd_argument((*untrusted_sig_path_ptr) + 8)) < 3)
+            errx(1, "--verify signature file descriptor must be 3 or more");
     }
-    // check if not already in list
-    for (i = 0; i < *list_count; i++) {
-        if (list[i] == cur_fd)
-            break;
-    }
-
-    if (i == *list_count)
-        list[(*list_count)++] = cur_fd;
+    add_fd_to_list(cur_fd, input_list, input_list_count, output_list, output_list_count);
 }
 
 /* This code is taken from the GUI daemon */
@@ -247,7 +249,7 @@ static void sanitize_string_from_vm(unsigned char *untrusted_s)
 
 int parse_options(int argc, char *untrusted_argv[], int *input_fds,
         int *input_fds_count, int *output_fds,
-        int *output_fds_count, int is_client)
+        int *output_fds_count)
 {
     int opt, command = 0;
     int longindex;
@@ -287,6 +289,22 @@ int parse_options(int argc, char *untrusted_argv[], int *input_fds,
         { "show-primary-key-only", true, true, false },
         { NULL, false, false, false },
     };
+    static struct listopt const allowed_export_options[] = {
+        { "export-local-sigs", true, true, false },
+        { "export-attributes", true, true, false },
+        { "export-sensitive-revkeys", true, true, false },
+        { "export-clean", true, true, false },
+        { "export-minimal", true, true, false },
+        { "export-dane", false, true, false },
+        { "backup", true, true, false },
+        { "include-local-sigs", true, true, false },
+        { "include-attributes", true, true, false },
+        { "include-sensitive-revkeys", true, true, false },
+        { "export-unusable-sigs", true, true, false },
+        { "export-clean-sigs", true, true, false },
+        { "export-clean-uids", true, true, false },
+        { NULL, false, false, false },
+    };
 
     *input_fds_count = 0;
     *output_fds_count = 0;
@@ -301,6 +319,9 @@ int parse_options(int argc, char *untrusted_argv[], int *input_fds,
     input_fds[(*input_fds_count)++] = 0;	//stdin
     output_fds[(*output_fds_count)++] = 1;	//stdout
     output_fds[(*output_fds_count)++] = 2;	//stderr
+    if (ioctl(2, is_client ? FIONCLEX : FIOCLEX) ||
+        ioctl(0, FIOCLEX) || ioctl(1, FIOCLEX))
+        errx(1, "File descriptor 0, 1, or 2 is bad");
 
     for (int i = 0; i < argc; ++i) {
         if (!untrusted_argv[i])
@@ -377,18 +398,18 @@ int parse_options(int argc, char *untrusted_argv[], int *input_fds,
             i++;
         }
         if (opt == opt_status_fd) {
-            add_arg_to_fd_list(output_fds, output_fds_count);
+            add_arg_to_fd_list(output_fds, output_fds_count, input_fds, input_fds_count);
         } else if (opt == opt_logger_fd) {
-            add_arg_to_fd_list(output_fds, output_fds_count);
+            add_arg_to_fd_list(output_fds, output_fds_count, input_fds, input_fds_count);
         } else if (opt == opt_attribute_fd) {
-            add_arg_to_fd_list(output_fds, output_fds_count);
+            add_arg_to_fd_list(output_fds, output_fds_count, input_fds, input_fds_count);
 #if 0
         } else if (opt == opt_passphrase_fd) {
             // this is senseless to enter password for private key in the source vm
-            add_arg_to_fd_list(input_fds, input_fds_count);
+            add_arg_to_fd_list(input_fds, input_fds_count, output_fds, output_fds_count);
 #endif
         } else if (opt == opt_command_fd) {
-            add_arg_to_fd_list(input_fds, input_fds_count);
+            add_arg_to_fd_list(input_fds, input_fds_count, output_fds, output_fds_count);
         } else if (opt == opt_verify) {
             mode_verify = 1;
         } else if (opt == 'o') {
@@ -397,9 +418,11 @@ int parse_options(int argc, char *untrusted_argv[], int *input_fds,
                 exit(1);
             }
         } else if (opt == opt_list_options) {
-            sanitize_list_or_verify_options(allowed_list_options, "list");
+            sanitize_option_list(allowed_list_options, "list");
         } else if (opt == opt_verify_options) {
-            sanitize_list_or_verify_options(allowed_verify_options, "verify");
+            sanitize_option_list(allowed_verify_options, "verify");
+        } else if (opt == opt_export_options) {
+            sanitize_option_list(allowed_export_options, "export");
         }
     }
     // Only allow key IDs to begin with '-' if the options list was terminated by '--',
@@ -421,7 +444,9 @@ int parse_options(int argc, char *untrusted_argv[], int *input_fds,
         optind = argc;
     }
     if (mode_verify && optind < argc) {
-        handle_opt_verify(untrusted_argv[optind], input_fds, input_fds_count, is_client);
+        handle_opt_verify(untrusted_argv + optind,
+                          input_fds, input_fds_count,
+                          output_fds, output_fds_count);
         /* the first path already processed */
         optind++;
     }
@@ -461,7 +486,7 @@ int prepare_pipes_and_run(const char *run_file, char **run_argv, int *input_fds,
         int input_fds_count, int *output_fds,
         int output_fds_count)
 {
-    int i;
+    int i, null_fd;
     pid_t pid;
     int pipes_in[MAX_FDS][2];
     int pipes_out[MAX_FDS][2];
@@ -473,19 +498,15 @@ int prepare_pipes_and_run(const char *run_file, char **run_argv, int *input_fds,
     sigaddset(&chld_set, SIGCHLD);
     if (input_fds_count > MAX_FDS || output_fds_count > MAX_FDS)
         abort();
-    else {
-        int const null_fd = open("/dev/null", O_RDONLY | O_NOCTTY | O_CLOEXEC | O_NOFOLLOW);
-        if (null_fd == -1) {
-            perror("open /dev/null");
-            exit(1);
-        }
-        for (i = 0; i < input_fds_count; ++i)
-            dup_over_fd(null_fd, input_fds[i]);
-        for (i = 0; i < output_fds_count; ++i)
-            dup_over_fd(null_fd, output_fds[i]);
-        close(null_fd);
-    }
-
+    null_fd = open("/dev/null", O_RDONLY | O_NOCTTY | O_CLOEXEC | O_NOFOLLOW);
+    if (null_fd < 0)
+        err(1, "open /dev/null");
+    for (i = 0; i < input_fds_count; ++i)
+        dup_over_fd(null_fd, input_fds[i]);
+    for (i = 0; i < output_fds_count; ++i)
+        dup_over_fd(null_fd, output_fds[i]);
+    // do not close null_fd yet; it could be one of the file descriptors
+    // to pass to GnuPG
     for (i = 0; i < input_fds_count; i++) {
         if (pipe2(pipes_in[i], O_CLOEXEC) < 0) {
             perror("pipe");
@@ -502,8 +523,10 @@ int prepare_pipes_and_run(const char *run_file, char **run_argv, int *input_fds,
         // multiplexer reads from gpg through this fd
         pipes_out_for_multiplexer[i] = pipes_out[i][0];
     }
+    // now that the pipes are created, null_fd can be closed safely
+    close(null_fd);
 
-    setup_sigchld(false);
+    setup_sigchld();
 
     switch (pid = fork()) {
         case -1:
