@@ -42,6 +42,8 @@ enum {
     opt_encrypt_to,
     opt_exit_on_status_write_error,
     opt_export,
+    opt_export_options,
+    opt_export_ownertrust,
     opt_export_ssh,
     opt_fingerprint,
     opt_fixed_list_mode,
@@ -86,6 +88,7 @@ enum {
     opt_s2k_mode,
     opt_sender,
     opt_set_filename,
+    opt_show_session_key,
     opt_sig_notation,
     opt_skip_hidden_recipients,
     opt_status_fd,
@@ -109,7 +112,7 @@ enum {
 
 int parse_options(int argc, char *argv[], int *input_fds,
         int *input_fds_count, int *output_fds,
-        int *output_fds_count, int is_client);
+        int *output_fds_count);
 int prepare_pipes_and_run(const char *run_file, char **run_argv, int *input_fds,
         int input_fds_count, int *output_fds,
         int output_fds_count);
@@ -148,6 +151,8 @@ static const int gpg_allowed_options[] = {
     opt_encrypt_to,
     opt_exit_on_status_write_error,
     opt_export,
+    opt_export_options,
+    opt_export_ownertrust,
     opt_export_ssh,
     opt_fingerprint,
     opt_fixed_list_mode,
@@ -187,6 +192,7 @@ static const int gpg_allowed_options[] = {
     opt_s2k_mode,
     opt_sender,
     opt_set_filename,
+    opt_show_session_key,
     opt_sig_notation,
     opt_skip_hidden_recipients,
     opt_status_fd,
@@ -228,6 +234,7 @@ static const struct gpg_command_opt gpg_commands[] = {
     {opt_clear_sign, false},
     {opt_export, true},
     {opt_export_ssh, true},
+    {opt_export_ownertrust, false},
     {opt_fingerprint, true},
     {opt_list_config, false},
     {opt_list_sigs, true},
@@ -270,6 +277,8 @@ static const struct option gpg_long_options[] = {
     {"exit-on-status-write-error", 0, 0, opt_exit_on_status_write_error},
     {"export", 0, 0, opt_export},
     {"export-ssh-key", 0, 0, opt_export_ssh},
+    {"export-options", 1, 0, opt_export_options},
+    {"export-ownertrust", 0, 0, opt_export_ownertrust},
     {"fingerprint", 0, 0, opt_fingerprint},
     {"fixed-list-mode", 0, 0, opt_fixed_list_mode},
     {"force-mdc", 0, 0, opt_force_mdc},
@@ -313,6 +322,7 @@ static const struct option gpg_long_options[] = {
     {"pgp6", 0, 0, opt_pgp6},
     {"pgp7", 0, 0, opt_pgp7},
     {"pgp8", 0, 0, opt_pgp8},
+    {"quiet", 0, 0, 'q'},
     {"recipient", 1, 0, 'r'},
     {"rfc2440", 0, 0, opt_rfc2440},
     {"rfc4880", 0, 0, opt_rfc4880},
@@ -320,6 +330,7 @@ static const struct option gpg_long_options[] = {
     {"s2k-count", 1, 0, opt_s2k_count},
     {"s2k-digest-algo", 1, 0, opt_s2k_digest_algo},
     {"s2k-mode", 1, 0, opt_s2k_mode},
+    {"show-session-key", 0, 0, opt_show_session_key},
     {"sender", 1, 0, opt_sender},
     {"sig-notation", 1, 0, opt_sig_notation},
     {"sign", 0, 0, 's'},
@@ -354,6 +365,8 @@ struct listopt {
     bool const allowed, allowed_negated, has_argument;
 };
 
-void sanitize_list_or_verify_options(const struct listopt *p, const char *msg);
+void sanitize_option_list(const struct listopt *p, const char *msg);
+
+extern const bool is_client;
 
 #endif				/* _GPG_H */
